@@ -45,16 +45,11 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
     if aaXf_orig.shape[0] != aaXm_orig.shape[0]:
         raise ValueError ("Inputs should have the same size.")
 
-    #convert weights into array
     N = aaXf_orig.shape[0]
-    if (aWeights == None) or (len(aWeights) == 0):
-        aWeights = np.full((N,1),1.0)
-    else:
-        #reshape so multiplications are done column-wise
-        aWeights = np.array(aWeights).reshape(N,1)
-
     # Find the center of mass of each object:
     """ # old code (using for-loops)
+    if (aWeights == None) or (len(aWeights) == 0):
+        aWeights = np.full(N, 1.0)
     aCenter_f = np.zeros(3)
     aCenter_m = np.zeros(3)
     sum_weights = 0.0
@@ -65,9 +60,15 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
         sum_weights += aWeights[n]
     """
     # new code (avoiding for-loops)
+    #convert weights into array
+    if (aWeights == None) or (len(aWeights) == 0):
+        aWeights = np.full((N,1),1.0)
+    else:
+        #reshape aWeights so multiplications are done column-wise
+        aWeights = np.array(aWeights).reshape(N,1)
     aCenter_f = np.sum(aaXf_orig * aWeights, axis=0)
     aCenter_m = np.sum(aaXm_orig * aWeights, axis=0)
-    sum_weights = np.sum(aWeights, axis=0)
+    sum_weights = np.sum(aWeights)
 
     # Subtract the centers-of-mass from the original coordinates for each object
     """ # old code (using for-loops)
@@ -141,7 +142,8 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
 
     # Calculate "p".  
     # "p" contains the optimal rotation (in backwards-quaternion format)
-    # (First specify the default value for p)
+    # (Note: A discussion of various quaternion conventions is included below.)
+    # First, specify the default value for p:
     p = np.zeros(4)
     p[3] = 1.0           # p = [0,0,0,1]    default value
     pPp = 0.0            # = p^T * P * p    (zero by default)
@@ -192,6 +194,13 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
     aaRotate[0][2] = 2*(p[0]*p[2] + p[1]*p[3]);
     aaRotate[2][0] = 2*(p[0]*p[2] - p[1]*p[3]);
 
+    # Alternatively, in modern python versions, this code also works:
+    """
+    from scipy.spatial.transform import Rotation as R
+    the_rotation = R.from_quat(p)
+    aaRotate = the_rotation.as_matrix()
+    """
+
     # Optional: Decide the scale factor, c
     c = 1.0   # by default, don't rescale the coordinates
     if allow_rescale and (not singular):
@@ -203,6 +212,7 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
                 Waxaixai += aWeights[a] * aaXm[a][i] * aaXm[a][i]
                 WaxaiXai += aWeights[a] * aaXm[a][i] * aaXf[a][i]
         """
+        # new code (avoiding for-loops)
         Waxaixai = np.sum(aWeights * aaXm ** 2)
         WaxaiXai = np.sum(aWeights * aaXf ** 2)
 
@@ -221,6 +231,7 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
     if sum_sqr_dist < 0.0: #(edge case due to rounding error)
         sum_sqr_dist = 0.0
     """
+    # new code (avoiding for-loops)
     E0 = np.sum((aaXf - c*aaXm)**2)
     sum_sqr_dist = max(0, E0 - c * 2.0 * pPp)
 
@@ -251,12 +262,17 @@ def Superpose3D(aaXf_orig,   # <-- coordinates for the "frozen" object
     aTranslate = aCenter_f - np.matmul(c*aaRotate,aCenter_m).T.reshape(3,)
 
     if report_quaternion: # does the caller want the quaternion?
+        # The p array is a quaternion that uses this convention:
+        #https://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.transform.Rotation.from_quat.html
+        # However it seems that the following convention is much more popular:
+        # https://en.wikipedia.org/wiki/Quaternions_and_spatial_rotation
+        # https://mathworld.wolfram.com/Quaternion.html
+        # So I return "q" (a version of "p" using the more popular convention).
         q = np.empty(4)
-        q[0] = p[3]  # Note: The "p" variable is not a quaternion in the
-        q[1] = p[0]  #       conventional sense because its elements
-        q[2] = p[1]  #       are in the wrong order.  I correct for that here.
-        q[3] = p[2]  #       "q" is the quaternion correspond to rotation R
+        q[0] = p[3]
+        q[1] = p[0]
+        q[2] = p[1]
+        q[3] = p[2]
         return rmsd, q, aTranslate, c
     else:
         return rmsd, aaRotate, aTranslate, c
-
